"""
Type annotations for freetier service client.

[Open documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/)

Usage::

    ```python
    from aiobotocore.session import get_session
    from types_aiobotocore_freetier.client import FreeTierClient

    session = get_session()
    async with session.create_client("freetier") as client:
        client: FreeTierClient
    ```
"""

import sys
from typing import Any, Dict, Mapping, Type

from aiobotocore.client import AioBaseClient
from botocore.client import ClientMeta

from .paginator import GetFreeTierUsagePaginator
from .type_defs import ExpressionTypeDef, GetFreeTierUsageResponseTypeDef

if sys.version_info >= (3, 12):
    from typing import Literal
else:
    from typing_extensions import Literal

__all__ = ("FreeTierClient",)

class BotocoreClientError(Exception):
    MSG_TEMPLATE: str

    def __init__(self, error_response: Mapping[str, Any], operation_name: str) -> None:
        self.response: Dict[str, Any]
        self.operation_name: str

class Exceptions:
    ClientError: Type[BotocoreClientError]
    InternalServerException: Type[BotocoreClientError]
    ThrottlingException: Type[BotocoreClientError]
    ValidationException: Type[BotocoreClientError]

class FreeTierClient(AioBaseClient):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Client)
    [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/)
    """

    meta: ClientMeta

    @property
    def exceptions(self) -> Exceptions:
        """
        FreeTierClient exceptions.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Client.exceptions)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/#exceptions)
        """

    def can_paginate(self, operation_name: str) -> bool:
        """
        Check if an operation can be paginated.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Client.can_paginate)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/#can_paginate)
        """

    async def close(self) -> None:
        """
        Closes underlying endpoint connections.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Client.close)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/#close)
        """

    async def generate_presigned_url(
        self,
        ClientMethod: str,
        Params: Mapping[str, Any] = ...,
        ExpiresIn: int = 3600,
        HttpMethod: str = ...,
    ) -> str:
        """
        Generate a presigned url given a client, its method, and arguments.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Client.generate_presigned_url)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/#generate_presigned_url)
        """

    async def get_free_tier_usage(
        self, *, filter: "ExpressionTypeDef" = ..., maxResults: int = ..., nextToken: str = ...
    ) -> GetFreeTierUsageResponseTypeDef:
        """
        Returns a list of all Free Tier usage objects that match your filters.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Client.get_free_tier_usage)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/#get_free_tier_usage)
        """

    def get_paginator(
        self, operation_name: Literal["get_free_tier_usage"]
    ) -> GetFreeTierUsagePaginator:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Client.get_paginator)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/#get_paginator)
        """

    async def __aenter__(self) -> "FreeTierClient":
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Client)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/)
        """

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> Any:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Client)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/client/)
        """



"""
Main interface for freetier service.

Usage::

    ```python
    from aiobotocore.session import get_session
    from types_aiobotocore_freetier import (
        Client,
        FreeTierClient,
        GetFreeTierUsagePaginator,
    )

    session = get_session()
    async with session.create_client("freetier") as client:
        client: FreeTierClient
        ...


    get_free_tier_usage_paginator: GetFreeTierUsagePaginator = client.get_paginator("get_free_tier_usage")
    ```
"""

from .client import FreeTierClient
from .paginator import GetFreeTierUsagePaginator

Client = FreeTierClient

__all__ = ("Client", "FreeTierClient", "GetFreeTierUsagePaginator")



"""
Type annotations for freetier service client paginators.

[Open documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/paginators/)

Usage::

    ```python
    from aiobotocore.session import get_session

    from types_aiobotocore_freetier.client import FreeTierClient
    from types_aiobotocore_freetier.paginator import (
        GetFreeTierUsagePaginator,
    )

    session = get_session()
    with session.create_client("freetier") as client:
        client: FreeTierClient

        get_free_tier_usage_paginator: GetFreeTierUsagePaginator = client.get_paginator("get_free_tier_usage")
    ```
"""

from typing import AsyncIterator, Generic, Iterator, TypeVar

from aiobotocore.paginate import AioPaginator
from botocore.paginate import PageIterator

from .type_defs import ExpressionTypeDef, GetFreeTierUsageResponseTypeDef, PaginatorConfigTypeDef

__all__ = ("GetFreeTierUsagePaginator",)

_ItemTypeDef = TypeVar("_ItemTypeDef")

class _PageIterator(Generic[_ItemTypeDef], PageIterator):
    def __iter__(self) -> Iterator[_ItemTypeDef]:
        """
        Proxy method to specify iterator item type.
        """

class GetFreeTierUsagePaginator(AioPaginator):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Paginator.GetFreeTierUsage)
    [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/paginators/#getfreetierusagepaginator)
    """

    def paginate(
        self, *, filter: ExpressionTypeDef = ..., PaginationConfig: PaginatorConfigTypeDef = ...
    ) -> AsyncIterator[GetFreeTierUsageResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/freetier.html#FreeTier.Paginator.GetFreeTierUsage.paginate)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/paginators/#getfreetierusagepaginator)
        """



"""
Type annotations for freetier service type definitions.

[Open documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_freetier/type_defs/)

Usage::

    ```python
    from types_aiobotocore_freetier.type_defs import DimensionValuesTypeDef

    data: DimensionValuesTypeDef = ...
    ```
"""

import sys
from typing import Any, Dict, List, Sequence

from .literals import DimensionType, MatchOptionType

if sys.version_info >= (3, 12):
    from typing import NotRequired
else:
    from typing_extensions import NotRequired
if sys.version_info >= (3, 12):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

__all__ = (
    "DimensionValuesTypeDef",
    "FreeTierUsageTypeDef",
    "PaginatorConfigTypeDef",
    "GetFreeTierUsageRequestRequestTypeDef",
    "ResponseMetadataTypeDef",
    "ExpressionTypeDef",
    "GetFreeTierUsageResponseTypeDef",
    "GetFreeTierUsageRequestGetFreeTierUsagePaginateTypeDef",
)

DimensionValuesTypeDef = TypedDict(
    "DimensionValuesTypeDef",
    {
        "Key": DimensionType,
        "MatchOptions": Sequence[MatchOptionType],
        "Values": Sequence[str],
    },
)
FreeTierUsageTypeDef = TypedDict(
    "FreeTierUsageTypeDef",
    {
        "actualUsageAmount": NotRequired[float],
        "description": NotRequired[str],
        "forecastedUsageAmount": NotRequired[float],
        "freeTierType": NotRequired[str],
        "limit": NotRequired[float],
        "operation": NotRequired[str],
        "region": NotRequired[str],
        "service": NotRequired[str],
        "unit": NotRequired[str],
        "usageType": NotRequired[str],
    },
)
PaginatorConfigTypeDef = TypedDict(
    "PaginatorConfigTypeDef",
    {
        "MaxItems": NotRequired[int],
        "PageSize": NotRequired[int],
        "StartingToken": NotRequired[str],
    },
)
GetFreeTierUsageRequestRequestTypeDef = TypedDict(
    "GetFreeTierUsageRequestRequestTypeDef",
    {
        "filter": NotRequired["ExpressionTypeDef"],
        "maxResults": NotRequired[int],
        "nextToken": NotRequired[str],
    },
)
ResponseMetadataTypeDef = TypedDict(
    "ResponseMetadataTypeDef",
    {
        "RequestId": str,
        "HTTPStatusCode": int,
        "HTTPHeaders": Dict[str, str],
        "RetryAttempts": int,
        "HostId": NotRequired[str],
    },
)
ExpressionTypeDef = TypedDict(
    "ExpressionTypeDef",
    {
        "And": NotRequired[Sequence[Dict[str, Any]]],
        "Dimensions": NotRequired[DimensionValuesTypeDef],
        "Not": NotRequired[Dict[str, Any]],
        "Or": NotRequired[Sequence[Dict[str, Any]]],
    },
)
GetFreeTierUsageResponseTypeDef = TypedDict(
    "GetFreeTierUsageResponseTypeDef",
    {
        "freeTierUsages": List[FreeTierUsageTypeDef],
        "nextToken": str,
        "ResponseMetadata": ResponseMetadataTypeDef,
    },
)
GetFreeTierUsageRequestGetFreeTierUsagePaginateTypeDef = TypedDict(
    "GetFreeTierUsageRequestGetFreeTierUsagePaginateTypeDef",
    {
        "filter": NotRequired[ExpressionTypeDef],
        "PaginationConfig": NotRequired[PaginatorConfigTypeDef],
    },
)
